@@ -3,25 +3,27 @@
 import asyncio
 import importlib
 import itertools
-from functools import partial
-from typing import Self
-import tomllib
 import json
 import os
 import sys
+import tomllib
+from functools import partial
+from typing import Any, Callable, Self
 
-from pyprland.common import PyprError, get_logger, init_logger
-from pyprland.ipc import get_event_stream, notify_error, notify_fatal, notify_info
+from pyprland.common import (
+    IPC_FOLDER,
+    get_logger,
+    init_logger,
+    merge,
+    run_interactive_program,
+)
+from pyprland.ipc import get_event_stream
 from pyprland.ipc import init as ipc_init
+from pyprland.ipc import notify_error, notify_fatal, notify_info
 from pyprland.plugins.interface import Plugin
+from pyprland.types import PyprError
 
-try:
-    CONTROL = f'/tmp/hypr/{ os.environ["HYPRLAND_INSTANCE_SIGNATURE"] }/.pyprland.sock'
-except KeyError:
-    print(
-        "This is a fatal error, assuming we are running documentation generation hence ignoring it"
-    )
-
+CONTROL = f"{IPC_FOLDER}/.pyprland.sock"
 OLD_CONFIG_FILE = "~/.config/hypr/pyprland.json"
 CONFIG_FILE = "~/.config/hypr/pyprland.toml"
 
@@ -36,8 +38,10 @@ class Pyprland:
     event_reader: asyncio.StreamReader
     stopped = False
     config: dict[str, dict] = {}
-    tasks: None | asyncio.TaskGroup = None
+    tasks: list[asyncio.Task] = []
+    tasks_group: None | asyncio.TaskGroup = None
     instance: Self | None = None
+    log_handler: Callable[[Plugin, str, tuple], None]
 
     @classmethod
     def _set_instance(cls, instance):
@@ -45,6 +49,8 @@ class Pyprland:
         cls.instance = instance
 
     def __init__(self):
+        self.pyprland_mutex_event = asyncio.Event()
+        self.pyprland_mutex_event.set()
         self.config = {}
         self.plugins: dict[str, Plugin] = {}
         self.log = get_logger()
@@ -55,36 +61,50 @@ class Pyprland:
         "Initializes the main structures"
         await self.load_config()  # ensure sockets are connected first
 
-    async def __open_config(self):
+    async def __open_config(self, config_filename=""):
         """Loads config file as self.config"""
-        if os.path.exists(OLD_CONFIG_FILE) and not os.path.exists(CONFIG_FILE):
-            self.log.warning("Consider changing your configuration to TOML format.")
+        if config_filename:
+            fname = os.path.expandvars(os.path.expanduser(config_filename))
+            if os.path.isdir(fname):
+                config: dict[str, Any] = {}
+                for toml_file in sorted(os.listdir(fname)):
+                    if not toml_file.endswith(".toml"):
+                        continue
+                    merge(config, self.__load_config_file(f"{fname}/{toml_file}"))
+                return config
+        else:
+            if os.path.exists(OLD_CONFIG_FILE) and not os.path.exists(CONFIG_FILE):
+                self.log.warning("Consider changing your configuration to TOML format.")
+            self.config.clear()
+            fname = os.path.expanduser(CONFIG_FILE)
 
-        self.config.clear()
-        fname = os.path.expanduser(CONFIG_FILE)
+        config = self.__load_config_file(fname)
+
+        if not config_filename:
+            for extra_config in list(config["pyprland"].get("include", [])):
+                merge(config, await self.__open_config(extra_config))
+            self.config.update(config)
+        return config
+
+    def __load_config_file(self, fname):
+        """Loads a configuration file and returns it as a dictionary"""
+        config = {}
         if os.path.exists(fname):
             self.log.info("Loading %s", fname)
-            try:
-                with open(fname, "rb") as f:
-                    self.config.update(tomllib.load(f))
-            except FileNotFoundError as e:
-                self.log.critical(
-                    "No config file found, create one at ~/.config/hypr/pyprland.json with a valid pyprland.plugins list"
-                )
-                raise PyprError() from e
+            with open(fname, "rb") as f:
+                try:
+                    config = tomllib.load(f)
+                except tomllib.TOMLDecodeError as e:
+                    self.log.critical("Problem reading %s: %s", fname, e)
+                    raise PyprError() from e
         elif os.path.exists(os.path.expanduser(OLD_CONFIG_FILE)):
             self.log.info("Loading %s", OLD_CONFIG_FILE)
-            try:
-                with open(os.path.expanduser(OLD_CONFIG_FILE), encoding="utf-8") as f:
-                    self.config.update(json.loads(f.read()))
-            except FileNotFoundError as e:
-                self.log.critical(
-                    "No config file found, create one at ~/.config/hypr/pyprland.json with a valid pyprland.plugins list"
-                )
-                raise PyprError() from e
+            with open(os.path.expanduser(OLD_CONFIG_FILE), encoding="utf-8") as f:
+                config = json.loads(f.read())
         else:
-            self.log.critical("No Config file found ! Please create %s", CONFIG_FILE)
+            self.log.critical("Config file not found! Please create %s", fname)
             raise PyprError()
+        return config
 
     async def _load_single_plugin(self, name, init) -> bool:
         "Load a single plugin, optionally calling `init`"
@@ -99,8 +119,8 @@ class Pyprland:
             if init:
                 await plug.init()
                 self.queues[name] = asyncio.Queue()
-                if self.tasks:
-                    self.tasks.create_task(self._plugin_runner_loop(name))
+                if self.tasks_group:
+                    self.tasks_group.create_task(self._plugin_runner_loop(name))
             self.plugins[name] = plug
         except ModuleNotFoundError as e:
             self.log.error("Unable to locate plugin called '%s'", name)
@@ -130,14 +150,17 @@ class Pyprland:
             if init:
                 try:
                     await self.plugins[name].load_config(self.config)
-                    await self.plugins[name].on_reload()
-                    self.plugins[name].log.info("configured")
+                    await asyncio.wait_for(self.plugins[name].on_reload(), timeout=5.0)
+                except asyncio.TimeoutError:
+                    self.plugins[name].log.info("timed out on reload")
                 except PyprError:
                     raise
                 except Exception as e:
                     await notify_info(f"Error initializing plugin {name}: {e}")
                     self.log.error("Error initializing plugin %s:", name, exc_info=True)
                     raise PyprError() from e
+                else:
+                    self.plugins[name].log.info("configured")
         if init_pyprland:
             plug = self.plugins["pyprland"]
             plug.set_commands(reload=self.load_config)  # type: ignore
@@ -150,14 +173,10 @@ class Pyprland:
         await self.__open_config()
         assert self.config
         await self.__load_plugins_config(init=init)
-        if self.config["pyprland"].get("colored_handlers_log", True):
-            self.log_handler = (  # pylint: disable=attribute-defined-outside-init
-                self.colored_log_handler
-            )
-        else:
-            self.log_handler = (  # pylint: disable=attribute-defined-outside-init
-                self.plain_log_handler
-            )
+        colored_logs = self.config["pyprland"].get("colored_handlers_log", True)
+        self.log_handler = (
+            self.colored_log_handler if colored_logs else self.plain_log_handler
+        )  # pylint: disable=attribute-defined-outside-init
 
     def plain_log_handler(self, plugin, name, params):
         "log a handler method without color"
@@ -196,7 +215,8 @@ class Pyprland:
                 if plugin == "pyprland":
                     await task()
                 else:
-                    await self.queues[plugin.name].put(task)
+                    if not plugin.aborted:
+                        await self.queues[plugin.name].put(task)
         if notify and not handled:
             await notify_info(f'"{notify}" not found')
         return handled
@@ -221,6 +241,13 @@ class Pyprland:
             # self.log.debug("[%s] %s", cmd, params.strip())
             await self._callHandler(full_name, params.rstrip("\n"))
 
+    async def exit_plugins(self):
+        "Exits all plugins"
+        for plugin in self.plugins.values():
+            if not plugin.aborted:
+                plugin.aborted = True
+                await asyncio.wait_for(plugin.exit(), timeout=2.0)
+
     async def read_command(self, reader, writer) -> None:
         "Receives a socket command"
         data = (await reader.readline()).decode()
@@ -229,10 +256,25 @@ class Pyprland:
             data = "exit"
         data = data.strip()
         if data == "exit":
+
+            async def _abort():
+                await self.exit_plugins()
+                # cancel the task group
+                for task in self.tasks:
+                    task.cancel()
+                writer.close()
+                await writer.wait_closed()
+                for q in self.queues.values():
+                    await q.put(None)
+                self.server.close()
+                # Ensure the process exits
+                await asyncio.sleep(1)
+                if os.path.exists(CONTROL):
+                    os.unlink(CONTROL)
+                os._exit(0)
+
             self.stopped = True
-            writer.close()
-            await writer.wait_closed()
-            self.server.close()
+            asyncio.create_task(_abort())
             return
         args = data.split(None, 1)
         if len(args) == 1:
@@ -252,35 +294,43 @@ class Pyprland:
 
     async def serve(self):
         "Runs the server"
-        try:
-            async with self.server:
-                await self.server.serve_forever()
-        finally:
-            await asyncio.gather(*(plugin.exit() for plugin in self.plugins.values()))
+        async with self.server:
+            await self.server.wait_closed()
 
     async def _plugin_runner_loop(self, name):
         "Runs tasks for a given plugin indefinitely"
         q = self.queues[name]
+        is_pyprland = name == "pyprland"
 
         while not self.stopped:
+            if not is_pyprland:
+                await self.pyprland_mutex_event.wait()
             try:
                 task = await q.get()
+                if task is None:
+                    return
+                if is_pyprland:
+                    self.pyprland_mutex_event.clear()
             except RuntimeError as e:
                 self.log.error("Aborting [%s] loop: %s", name, e)
                 return
             try:
-                await task()
+                await asyncio.wait_for(task(), timeout=12.0)
+            except asyncio.TimeoutError:
+                self.log.error("Timeout running plugin %s::%s", name, task)
             except Exception as e:  # pylint: disable=W0718
                 self.log.error(
                     "Unhandled error running plugin %s::%s: %s", name, task, e
                 )
+            if is_pyprland and q.empty():
+                self.pyprland_mutex_event.set()
 
     async def plugins_runner(self):
         "Runs plugins' task using the created `tasks` TaskGroup attribute"
         async with asyncio.TaskGroup() as group:
-            self.tasks = group
+            self.tasks_group = group
             for name in self.plugins:
-                group.create_task(self._plugin_runner_loop(name))
+                self.tasks.append(group.create_task(self._plugin_runner_loop(name)))
 
     async def run(self):
         "Runs the server and the event listener"
@@ -336,44 +386,70 @@ async def run_daemon():
         print("Interrupted")
     except asyncio.CancelledError:
         manager.log.critical("cancelled")
-    finally:
+    else:
+        await manager.exit_plugins()
         events_writer.close()
         await events_writer.wait_closed()
         manager.server.close()
         await manager.server.wait_closed()
 
 
+def show_help(manager):
+    "show the documentation"
+
+    def format_doc(txt):
+        return txt.split("\n")[0]
+
+    print(
+        """Syntax: pypr [command]
+
+If the command is ommited, runs the daemon which will start every configured plugin.
+
+Available commands:
+"""
+    )
+    builtins_docs = {
+        "dumpjson": "Dumps the configuration in JSON format",
+        "edit": "Edit the configuration file",
+        "exit": "Exits the daemon",
+        "help": "Shows this help",
+        "version": "Shows the version",
+    }
+    for name, doc in builtins_docs.items():
+        print(f" {name:20s} {doc}")
+
+    for plug in manager.plugins.values():
+        for name in dir(plug):
+            if not name.startswith("run_"):
+                continue
+            fn = getattr(plug, name)
+            if callable(fn):
+                doc_txt = format_doc(fn.__doc__) if fn.__doc__ else "N/A"
+                print(f" {name[4:]:20s} {doc_txt} [{plug.name}]")
+
+
 async def run_client():
     "Runs the client (CLI)"
     manager = Pyprland()
 
     if sys.argv[1] == "version":
-        print("2.1.4-23")  # Automatically updated version
+        print("2.2.16")  # Automatically updated version
         return
 
+    if sys.argv[1] == "edit":
+        editor = os.environ.get("EDITOR", os.environ.get("VISUAL", "vi"))
+        filename = os.path.expanduser(CONFIG_FILE)
+        run_interactive_program(f'{editor} "{filename}"')
+        sys.argv[1] = "reload"
+
     if sys.argv[1] in ("--help", "-h", "help"):
         await manager.load_config(init=False)
+        return show_help(manager)
 
-        def format_doc(txt):
-            return txt.split("\n")[0]
-
-        print(
-            """Syntax: pypr [command]
-
-If the command is ommited, runs the daemon which will start every configured plugin.
-
-Available commands:
-"""
-        )
-        for plug in manager.plugins.values():
-            for name in dir(plug):
-                if name.startswith("run_"):
-                    fn = getattr(plug, name)
-                    if callable(fn):
-                        print(
-                            f" {name[4:]:20s} {format_doc(fn.__doc__) if fn.__doc__ else 'N/A'} [{plug.name}]"
-                        )
-
+    if sys.argv[1] in ("dumpjson"):
+        await manager.load_config(init=False)
+        # Dump manager.config in TOML format
+        print(json.dumps(manager.config, indent=2))
         return
 
     try:
@@ -439,7 +515,7 @@ If that's not the case, delete this file and run again.""",
         except Exception:  # pylint: disable=W0718
             log.critical("Unhandled exception:", exc_info=True)
         finally:
-            if invoke_daemon:
+            if invoke_daemon and os.path.exists(CONTROL):
                 os.unlink(CONTROL)
 
 
