@@ -13,22 +13,18 @@ __all__ = [
 
 import asyncio
 import json
-import os
-from typing import Any
-from logging import Logger
+import time
 from functools import partial
+from logging import Logger
+from typing import Any
 
-from .common import PyprError, get_logger
+from .common import IPC_FOLDER, get_logger
+from .types import ClientInfo, MonitorInfo, PyprError
 
 log: Logger | None = None
 
-try:
-    HYPRCTL = f'/tmp/hypr/{ os.environ["HYPRLAND_INSTANCE_SIGNATURE"] }/.socket.sock'
-    EVENTS = f'/tmp/hypr/{ os.environ["HYPRLAND_INSTANCE_SIGNATURE"] }/.socket2.sock'
-except KeyError:
-    print(
-        "This is a fatal error, assuming we are running documentation generation hence ignoring it"
-    )
+HYPRCTL = f"{IPC_FOLDER}/.socket.sock"
+EVENTS = f"{IPC_FOLDER}/.socket2.sock"
 
 
 async def notify(text, duration=3, color="ff1010", icon=-1, logger=None):
@@ -55,7 +51,7 @@ def retry_on_reset(func):
         exc = None
         for count in range(3):
             try:
-                return await func(*args, **kwargs)
+                return await func(*args, **kwargs, logger=logger)
             except ConnectionResetError as e:
                 exc = e
                 logger.warning("ipc connection problem, retrying...")
@@ -66,12 +62,25 @@ def retry_on_reset(func):
     return wrapper
 
 
+cached_responses: dict[str, list[Any]] = {
+    # <command name>: [expiration_date, payload, retention_time]
+    "monitors": [0, None, 0.03],
+    # "workspaces": [0, None, 0.1],
+    # "clients": [0, None, 0.02],
+}
+
+
 @retry_on_reset
 async def hyprctlJSON(
     command: str, logger=None
 ) -> list[dict[str, Any]] | dict[str, Any]:
     """Run an IPC command and return the JSON output."""
     logger = logger or log
+    now = time.time()
+    cached = command in cached_responses and cached_responses[command][0] > now
+    if cached:
+        logger.debug(f"{command} (CACHE HIT)")
+        return cached_responses[command][1]  # type: ignore
     logger.debug(command)
     try:
         ctl_reader, ctl_writer = await asyncio.open_unix_connection(HYPRCTL)
@@ -85,6 +94,9 @@ async def hyprctlJSON(
     await ctl_writer.wait_closed()
     ret = json.loads(resp)
     assert isinstance(ret, (list, dict))
+    if command in cached_responses:  # should fill the cache
+        cached_responses[command][0] = now + cached_responses[command][2]
+        cached_responses[command][1] = ret
     return ret
 
 
@@ -136,7 +148,7 @@ async def hyprctl(
     return r
 
 
-async def get_focused_monitor_props(logger=None, name=None) -> dict[str, Any]:
+async def get_focused_monitor_props(logger=None, name=None) -> MonitorInfo:
     """Returns focused monitor data if `name` is not defined, else use monitor's name
 
     Args:
@@ -160,42 +172,58 @@ async def get_focused_monitor_props(logger=None, name=None) -> dict[str, Any]:
     for monitor in await hyprctlJSON("monitors", logger=logger):
         assert isinstance(monitor, dict)
         if match_fn(monitor):
-            return monitor
+            return monitor  # type: ignore
     raise RuntimeError("no focused monitor")
 
 
 async def get_client_props(
-    addr: str | None = None, pid: int | None = None, cls: str | None = None, logger=None
-):
+    logger=None, match_fn=None, clients: list[ClientInfo] | None = None, **kw
+) -> ClientInfo | None:
     """
-    Returns client properties
+    Returns the properties of a client that matches the given `match_fn` (or default to equality) given the keyword arguments
+
+    Eg.
+        # will return the properties of the client with address "0x1234"
+        get_client_props(logger, addr="0x1234")
+
+        # will return the properties of the client with initialClass containing "fooBar"
+        get_client_props(logger, match_fn=lambda x, y: y in x), initialClass="fooBar")
 
     Args:
-        addr (str): address of the client (includes "0x" prefix)
-        pid (int): PID of the client
-        cls (str): class of the client
-        logger: logger to use in case of error
-    """
-    assert addr or pid or cls
 
-    prop_value: int | str
+        logger: logger to use in case of error
+        match_fn: function to match the client properties, takes 2 arguments (client_value, config_value)
+
+        Any other keyword argument will be used to match the client properties. Only one keyword argument is allowed.
+        `addr` aliases `address` and `cls` aliases `class`
+
+        Note: the address of the client must include the "0x" prefix
+    """
+    assert kw
+
+    addr = kw.get("addr")
+    klass = kw.get("cls")
 
     if addr:
         assert len(addr) > 2, "Client address is invalid"
         prop_name = "address"
         prop_value = addr
-    elif cls:
+    elif klass:
         prop_name = "class"
-        prop_value = cls
+        prop_value = klass
     else:
-        assert pid, "Client pid is invalid"
-        prop_name = "pid"
-        prop_value = pid
+        prop_name, prop_value = next(iter(kw.items()))
 
-    for client in await hyprctlJSON("clients", logger=logger):
+    if match_fn is None:
+
+        def match_fn(value1, value2):
+            return value1 == value2
+
+    for client in clients or await hyprctlJSON("clients", logger=logger):
         assert isinstance(client, dict)
-        if client.get(prop_name) == prop_value:
-            return client
+        if match_fn(client.get(prop_name), prop_value):
+            return client  # type: ignore
+    return None
 
 
 def init():
