@@ -4,8 +4,9 @@ from collections import defaultdict
 from copy import deepcopy
 from typing import Any, cast
 
+from ..common import CastBoolMixin, is_rotated, state
+from ..types import MonitorInfo
 from .interface import Plugin
-from ..common import CastBoolMixin
 
 
 def trim_offset(monitors):
@@ -39,8 +40,12 @@ def get_XY(place, main_mon, other_mon):
     align_x = False
     scaled_m_w = int(main_mon["width"] / main_mon["scale"])
     scaled_m_h = int(main_mon["height"] / main_mon["scale"])
+    if is_rotated(main_mon):
+        scaled_m_w, scaled_m_h = scaled_m_h, scaled_m_w
     scaled_om_w = int(other_mon["width"] / other_mon["scale"])
     scaled_om_h = int(other_mon["height"] / other_mon["scale"])
+    if is_rotated(other_mon):
+        scaled_om_w, scaled_om_h = scaled_om_h, scaled_om_w
     if place.startswith("top"):
         x = other_mon["x"]
         y = other_mon["y"] - scaled_m_h
@@ -92,19 +97,22 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
 
     async def on_reload(self):
         self._clear_mon_by_pat_cache()
+        monitors = await self.hyprctlJSON("monitors")
         if self.cast_bool(self.config.get("startup_relayout"), True):
-            await self.run_relayout()
+            await self.run_relayout(monitors)
+
+        for mon in state.monitors:
+            await self._hotplug_command(name=mon, monitors=monitors)
 
     # Command
 
-    async def run_relayout(
-        self,
-    ):
+    async def run_relayout(self, monitors: list[MonitorInfo] | None = None):
         "Recompute & apply every monitors's layout"
 
         self._clear_mon_by_pat_cache()
 
-        monitors = cast(list[dict], await self.hyprctlJSON("monitors"))
+        if monitors is None:
+            monitors = cast(list[MonitorInfo], await self.hyprctlJSON("monitors"))
 
         cleaned_config = self.resolve_names(monitors)
         if cleaned_config:
@@ -123,20 +131,34 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
                 resolution = f"{this_mon['width']}x{this_mon['height']}@{this_mon['refreshRate']}"
                 scale = this_mon["scale"]
                 position = f"{monitor['x']}x{monitor['y']}"
+                transform = this_mon["transform"]
 
                 await self.hyprctl(
-                    f"monitor {name},{resolution},{position},{scale}", "keyword"
+                    f"monitor {name},{resolution},{position},{scale},transform,{transform}",
+                    "keyword",
                 )
 
     # Event handlers
 
-    async def event_monitoradded(self, _) -> None:
+    async def event_monitoradded(self, name) -> None:
         "Triggers when a monitor is plugged"
         await asyncio.sleep(self.config.get("new_monitor_delay", 1.0))
-        await self.run_relayout()
+        monitors = await self.hyprctlJSON("monitors")
+        await self._hotplug_command(monitors, name)
+        await self.run_relayout(monitors)
 
     # Utils
 
+    async def _hotplug_command(self, monitors: list[MonitorInfo], name: str):
+        "Run the hotplug command for the monitor"
+        monitors_by_descr = {m["description"]: m for m in monitors}
+        monitors_by_name = {m["name"]: m for m in monitors}
+        for descr, command in self.config.get("hotplug_commands", {}).items():
+            mon = self._get_mon_by_pat(descr, monitors_by_descr, monitors_by_name)
+            if mon and mon["name"] == name:
+                await asyncio.create_subprocess_shell(command)
+                break
+
     def _clear_mon_by_pat_cache(self):
         "clear the cache"
         self._mon_by_pat_cache = {}
@@ -223,10 +245,10 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
             mon = self._get_mon_by_pat(descr1, monitors_by_descr, plugged_monitors)
             if not mon:
                 continue
-            name1 = mon["name"]
-            if name1 not in plugged_monitors:
+            name = mon["name"]
+            if name not in plugged_monitors:
                 continue
-            cleaned_config[name1] = {}
+            cleaned_config[name] = {}
             for position, descr_list in placement.items():
                 if isinstance(descr_list, str):
                     descr_list = [descr_list]
@@ -236,7 +258,7 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
                     if r:
                         resolved.append(r["name"])
                 if resolved:
-                    cleaned_config[name1][clean_pos(position)] = [
+                    cleaned_config[name][clean_pos(position)] = [
                         r for r in resolved if r in plugged_monitors
                     ]
         return cleaned_config
