@@ -1,11 +1,12 @@
 " Add system notifications based on journal logs "
-import re
 import asyncio
-from typing import cast
+import re
 from copy import deepcopy
-from .interface import Plugin
-from ..common import apply_filter
+from typing import cast
+
 from ..adapters.colors import convert_color
+from ..common import apply_filter
+from .interface import Plugin
 
 builtin_parsers = {
     "journal": [
@@ -60,11 +61,14 @@ class Extension(Plugin):
             self.tasks.append(asyncio.create_task(self.start_source(props)))
 
     async def exit(self):
-        "empty exit function"
+        "exit function"
         self.running = False
         for task in self.tasks:
             task.cancel()
-            await task
+            try:
+                await task
+            except asyncio.CancelledError:
+                pass
         for source in self.sources.values():
             if source.pid is not None:
                 source.kill()
